import json, shortuuid
from urllib.parse import quote, urlsplit, urlunsplit
import os, gzip, glob, zipfile
from cdxj_indexer.main import CDXJIndexer
from warcio.timeutils import iso_date_to_timestamp, timestamp_to_iso_date
from boilerpy3 import extractors
from wacz.util import support_hash_file, now, WACZ_VERSION

HTML_MIME_TYPES = ("text/html", "application/xhtml", "application/xhtml+xml")


# ============================================================================
class WACZIndexer(CDXJIndexer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.pages = {}
        self.extra_page_lists = {}
        self.title = ""
        self.desc = ""
        self.has_text = False
        self.main_url = kwargs.pop("main_url", "")
        self.main_ts = kwargs.pop("main_ts", "")
        self.hash_type = kwargs.pop("hash_type", "")

        # If the user has specified a hash type use that otherwise default to sha256
        if self.hash_type == None:
            self.hash_type = "sha256"

        self.passed_pages_dict = kwargs.pop("passed_pages_dict", "")

        if self.main_url != None and self.main_url != "":
            self.main_url_flag = False
            self.main_ts_flag = False
        # if url is missing path segment, ensure it is set to '/'
        try:
            parts = list(urlsplit(self.main_url))
            if not parts[2]:
                parts[2] = "/"
                self.main_url = urlunsplit(parts)
        except:
            pass

        self.detect_pages = kwargs.get("detect_pages")
        self.extract_text = kwargs.get("extract_text")

        self.referrers = set()

    def process_index_entry(self, it, record, *args):
        type_ = record.rec_headers.get("WARC-Type")
        if type_ == "warcinfo":
            self.parse_warcinfo(record)

        elif type_ in CDXJIndexer.DEFAULT_RECORDS:
            if type_ in ("response" "resource"):
                self.check_pages_and_text(record)

            super().process_index_entry(it, record, *args)

    def process_all(self):
        super().process_all()

        if self.detect_pages:
            to_delete = [
                id_
                for id_, value in self.pages.items()
                if value["url"] not in self.referrers
            ]
            for delete in to_delete:
                del self.pages[delete]

            if self.passed_pages_dict == {}:
                print("Num Pages Detected: {0}".format(len(self.pages)))

        if (
            hasattr(self, "main_url_flag")
            and hasattr(self, "main_ts_flag")
            and self.main_url_flag == False
            and self.main_ts_flag == False
        ):
            raise ValueError(
                "ts %s not found in index with %s" % (self.main_ts, self.main_url)
            )

        if hasattr(self, "main_url_flag") and self.main_url_flag == False:
            raise ValueError("Url %s not found in index" % (self.main_url))

    def _do_write(self, urlkey, ts, index, out):
        if self.detect_pages:
            self.detect_page(ts, index)

        super()._do_write(urlkey, ts, index, out)

    def detect_page(self, ts, index):
        referrer = index.get("referrer")
        if referrer:
            self.referrers.add(referrer)

    def _read_record(self, record):
        if hasattr(record, "buffered_stream"):
            content = record.buffered_stream.read()
        else:
            content = record.content_stream().read()

        return content

    def parse_warcinfo(self, record):
        """Parse WARC information.
        :param record: WARC information
        :returns: WARC information or None
        :rtype: dict or None
        """
        warcinfo = {}
        warcinfo_buff = self._read_record(record)
        warcinfo_buff = warcinfo_buff.decode("utf-8")
        metadata = None
        for line in warcinfo_buff.rstrip().split("\n"):
            parts = line.split(":", 1)
            if parts[0] == "json-metadata":
                metadata = json.loads(parts[1])
            elif len(parts) == 2:
                warcinfo[parts[0]] = parts[1].strip()

        if not metadata or "type" not in metadata:
            return

        if metadata["type"] == "collection":
            self.title = metadata.get("title", "")
            self.desc = metadata.get("desc", "")
            lists = metadata.get("lists")
            if lists:
                self.extract_page_lists(lists)

        # Don't add the record to the self.pages if were evaluating passed in pages
        elif metadata["type"] == "recording" and self.passed_pages_dict == {}:
            pages = metadata.get("pages", [])
            for page in pages:
                id_ = page["timestamp"] + "/" + page["url"]
                self.pages[id_] = page

        self.detect_pages = False

    def extract_page_lists(self, lists):
        for pagelist in lists:
            pagelist_header = {}
            # unique id for this page list, will also be the filename
            if "slug" in pagelist:
                uid = pagelist["slug"]
            else:
                uid = shortuuid.uuid()

            text_list = list(
                self.serialize_json_pages(
                    pages=pagelist["bookmarks"],
                    id=uid,
                    title=pagelist.get("title"),
                    desc=pagelist.get("desc"),
                )
            )

            self.extra_page_lists[uid] = text_list

    def check_pages_and_text(self, record):
        url = record.rec_headers.get("WARC-Target-URI")
        date = record.rec_headers.get("WARC-Date")
        ts = iso_date_to_timestamp(date)
        id_ = ts + "/" + url
        matched_id = ""
        # Check for both a matching url/ts and url entry
        if id_ in self.passed_pages_dict:
            matched_id = id_
        if url in self.passed_pages_dict:
            matched_id = url
        # If we find a match build a record
        if matched_id != "":
            self.pages[matched_id] = {"timestamp": ts, "url": url, "title": url}
            # Add title and text if they've been provided
            if "title" in self.passed_pages_dict[matched_id]:
                self.pages[matched_id]["title"] = self.passed_pages_dict[matched_id][
                    "title"
                ]
            if "text" in self.passed_pages_dict[matched_id]:
                self.pages[matched_id]["text"] = self.passed_pages_dict[matched_id][
                    "text"
                ]
            # Delete the entry from our pages_dict so we can't match it again
            del self.passed_pages_dict[matched_id]

        if (
            self.main_url
            and self.main_url == url
            and self.main_ts
            and self.main_ts == ts
        ):
            self.main_ts_flag = True
            self.main_url_flag = True
            print("Found Main Url: {0}".format(url))
            print("Found Main ts: {0}".format(ts))
            # If were not relying on passed in pages we want to add all records to the self.pages object
            if self.passed_pages_dict == {}:
                self.pages[id_] = {"timestamp": ts, "url": url, "title": url}
        if self.main_url and self.main_url == url and self.main_ts == None:
            self.main_url_flag = True
            print("Found Main Url: {0}".format(url))
            if id_ not in self.pages:
                self.pages[id_] = {"timestamp": ts, "url": url, "title": url}

        mime = self.get_record_mime_type(record)

        if mime not in HTML_MIME_TYPES:
            return

        status = record.http_headers.get_statuscode()
        if record.http_headers and status.startswith("3"):
            return

        if id_ not in self.pages:

            if self.detect_pages:
                self.pages[id_] = {"timestamp": ts, "url": url, "title": url}
            else:
                return

        # if not extracting text, then finish here
        if not self.extract_text:
            return

        content = self._read_record(record)
        if not content:
            return

        try:
            extractor = extractors.ArticleExtractor()

            content = content.decode("utf-8")

            doc = extractor.get_doc(content)

            curr_page = self.pages[id_]

            if doc.content:
                self.pages[id_]["text"] = doc.content
                self.has_text = True

            # only set title if unset, or set to url (default)
            # avoid overriding user-specified title, if any
            if doc.title and self.pages[id_].get("title", url) == url:
                self.pages[id_]["title"] = doc.title

        except Exception as e:
            # skip text extraction in case of errors
            print("Skipping, Text Extraction Failed For: " + url)
            print(e)

    def get_record_mime_type(self, record):
        if record.http_headers:
            # if the record has HTTP headers, use the Content-Type from those (eg. 'response' record)
            content_type = record.http_headers["Content-Type"]
        else:
            # otherwise, use the Content-Type from WARC headers
            content_type = record.rec_headers["Content-Type"]

        mime = content_type or ""
        return mime.split(";")[0]

    def write_page_list(self, wacz, filename, page_iter):
        pages_file = zipfile.ZipInfo(filename, now())
        pages_file.compress_type = zipfile.ZIP_DEFLATED

        with wacz.open(pages_file, "w") as pg_fh:
            for line in page_iter:
                pg_fh.write(line.encode("utf-8"))

    def serialize_json_pages(self, pages, id, title, desc=None, has_text=False):
        page_header = {"format": "json-pages-1.0", "id": id}

        if title:
            page_header["title"] = title

        if desc:
            page_header["description"] = desc

        if has_text:
            page_header["hasText"] = True

        yield json.dumps(page_header) + "\n"

        for line in pages:
            ts = timestamp_to_iso_date(line["timestamp"])
            page_title = line.get("title")

            uid = line.get("id") or line.get("page_id") or shortuuid.uuid()

            data = {"id": uid, "url": line["url"], "ts": ts}

            if page_title:
                data["title"] = page_title

            if "text" in line:
                data["text"] = line["text"]

            yield json.dumps(data) + "\n"

    def generate_metadata(self, res, wacz):
        package_dict = {}
        metadata = {}

        package_dict["profile"] = "data-package"
        package_dict["resources"] = []
        for i in range(0, len(wacz.infolist())):
            file = wacz.infolist()[i]
            package_dict["resources"].append({})
            package_dict["resources"][i]["path"] = file.filename
            with wacz.open(file, "r") as myfile:
                content = myfile.read()
                package_dict["resources"][i]["stats"] = {}
                package_dict["resources"][i]["stats"]["hash"] = support_hash_file(
                    self.hash_type, content
                )
                package_dict["resources"][i]["stats"]["bytes"] = len(content)
                package_dict["resources"][i]["hashing"] = self.hash_type

        # set optional metadata
        desc = res.desc or self.desc
        title = res.title or self.title

        if title:
            metadata["title"] = title

        if desc:
            metadata["desc"] = desc

        if self.main_url:
            metadata["mainPageURL"] = self.main_url
            if self.main_ts:
                metadata["mainPageTS"] = self.main_ts

        if res.date:
            metadata["mainPageTS"] = res.date

        package_dict["metadata"] = metadata
        package_dict["wacz_version"] = WACZ_VERSION

        return json.dumps(package_dict, indent=2)



import unittest, os, zipfile, sys, gzip, json, tempfile
from wacz.main import main
from frictionless import validate
from wacz.validate import Validation
from unittest.mock import patch

TEST_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures")


class TestWaczFormat(unittest.TestCase):
    @classmethod
    @patch("wacz.main.now")
    def setUpClass(self, mock_now):
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        self.tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(self.tmpdir.name, "valid_example_1.wacz"),
            ]
        )
        with zipfile.ZipFile(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(self.tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        self.validation_class_valid_1 = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )
        self.validation_class_invalid = Validation(
            os.path.join(TEST_DIR, "invalid_example_1.wacz")
        )

    def test_overall_command(self):
        self.assertTrue(
            (["validate", "-f", os.path.join(self.tmpdir.name, "valid_example_1.wacz")])
        )

    def test_check_indexes_valid(self):
        self.assertTrue(self.validation_class_valid_1.check_indexes())

    def test_check_compression_valid(self):
        self.assertTrue(self.validation_class_valid_1.check_compression())

    def test_frictionless_validate_valid_wacz(self):
        """Check that the frictionless validation feature identifies a valid wacz data package as valid"""
        # Use frictionless validation
        valid_1 = self.validation_class_valid_1.frictionless_validate()
        self.assertTrue(valid_1)

    def test_frictionless_validate_invalid_wacz(self):
        """Check that the frictionless validation feature identifies an invalid wacz data package as invalid"""
        # Use frictionless validation
        valid = self.validation_class_invalid.frictionless_validate()
        self.assertFalse(valid)

    def test_filepaths_invalid_wacz(self):
        """Correctly fail on a wacz with invalid files"""
        valid = self.validation_class_invalid.check_file_paths()
        self.assertFalse(valid)

    def test_filepaths_valid_wacz(self):
        """Correctly succeed on a wacz with valid files"""
        valid_1 = self.validation_class_valid_1.check_file_paths()
        self.assertTrue(valid_1)

    def test_hashes_valid_wacz(self):
        """Correctly succeed on a wacz with matching hashes"""
        valid_1 = self.validation_class_valid_1.check_file_hashes()
        self.assertTrue(valid_1)

    def test_hashes_invalid_wacz(self):
        """Correctly fail on a wacz with nonmatching hashes"""
        valid = self.validation_class_invalid.check_file_hashes()
        self.assertFalse(valid)

    def test_ability_to_detect_hash_md5(self):
        """Correctly identify the hash type of a file as md5"""
        tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(tmpdir.name, "valid_example_1.wacz"),
                "--hash-type",
                "md5",
            ]
        )
        with zipfile.ZipFile(
            os.path.join(tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        validation_class = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )
        valid = validation_class.detect_hash_type()
        self.assertEqual(valid, 0)
        valid = validation_class.hash_type
        self.assertEqual(valid, "md5")

    def test_ability_to_detect_hash_md5(self):
        """Correctly validate hashes and identify the type when no flag is set to md5"""
        tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(tmpdir.name, "valid_example_1.wacz"),
                "--hash-type",
                "sha256",
            ]
        )
        with zipfile.ZipFile(
            os.path.join(tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        validation_class = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )
        valid = validation_class.detect_hash_type()
        self.assertEqual(valid, 0)
        valid = validation_class.hash_type
        self.assertEqual(valid, "sha256")

    def test_ability_to_detect_hash_md5(self):
        """Correctly validate hashes and identify the type when flag is set to 256"""
        tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(tmpdir.name, "valid_example_1.wacz"),
                "--hash-type",
                "sha256",
            ]
        )
        with zipfile.ZipFile(
            os.path.join(tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        validation_class = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )
        valid = validation_class.detect_hash_type()
        self.assertEqual(valid, 0)
        valid = validation_class.hash_type
        self.assertEqual(valid, "sha256")

    def test_invalid_wacz_missing_datapackage(self):
        """Correctly validate hashes and identify the type when no flag is set"""
        tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(tmpdir.name, "valid_example_1.wacz"),
            ]
        )
        with zipfile.ZipFile(
            os.path.join(tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        os.remove(os.path.join(tmpdir.name, "unzipped_wacz_1/datapackage.json"))
        validation_class = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )

        valid = validation_class.check_required_contents()
        self.assertEqual(valid, 0)

    def test_invalid_wacz_missing_index(self):
        """Correctly fail on a wacz with no index"""
        tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(tmpdir.name, "valid_example_1.wacz"),
            ]
        )
        with zipfile.ZipFile(
            os.path.join(tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        os.remove(os.path.join(tmpdir.name, "unzipped_wacz_1/indexes/index.cdx.gz"))
        validation_class = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )
        valid = validation_class.check_required_contents()
        self.assertEqual(valid, 0)

    def test_invalid_wacz_missing_warc(self):
        """Correctly fail on a wacz with no warc file"""
        tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(tmpdir.name, "valid_example_1.wacz"),
            ]
        )
        with zipfile.ZipFile(
            os.path.join(tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        os.remove(
            os.path.join(tmpdir.name, "unzipped_wacz_1/archive/example-collection.warc")
        )
        validation_class = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )
        valid = validation_class.check_required_contents()
        self.assertEqual(valid, 0)

    def test_invalid_wacz_missing_pages(self):
        """Correctly fail on a wacz with no pages file"""
        tmpdir = tempfile.TemporaryDirectory()
        main(
            [
                "create",
                "-f",
                os.path.join(TEST_DIR, "example-collection.warc"),
                "-o",
                os.path.join(tmpdir.name, "valid_example_1.wacz"),
            ]
        )
        with zipfile.ZipFile(
            os.path.join(tmpdir.name, "valid_example_1.wacz"), "r"
        ) as zip_ref:
            zip_ref.extractall(os.path.join(tmpdir.name, "unzipped_wacz_1"))
            zip_ref.close()

        os.remove(os.path.join(tmpdir.name, "unzipped_wacz_1/pages/pages.jsonl"))
        validation_class = Validation(
            os.path.join(self.tmpdir.name, "valid_example_1.wacz")
        )
        valid = validation_class.check_required_contents()
        self.assertEqual(valid, 0)


if __name__ == "__main__":
    unittest.main()



import hashlib, datetime, json
from warcio.timeutils import iso_date_to_timestamp

WACZ_VERSION = "1.0.0"


def support_hash_file(hash_type, data):
    """Hashes the passed content using sha256 or md5"""
    if hash_type == "sha256":
        return hashlib.sha256(data).hexdigest()
    if hash_type == "md5":
        return hashlib.md5(data).hexdigest()


def construct_passed_pages_dict(passed_content):
    """Creates a dictionary of the passed pages with the url as the key or ts/url if ts is present and the title and text as the values if they have been passed"""
    passed_pages_dict = {}
    for i in range(0, len(passed_content)):
        # Skip the file's header if it's been set
        header = json.loads(passed_content[i])
        if "format" not in header:
            pages_dict = dict(header)

            # Set the default key as url
            key = "%s" % pages_dict["url"]

            # If timestamp is present overwrite the key to be 'ts/url'
            if "ts" in pages_dict:
                key = "%s/%s" % (
                    iso_date_to_timestamp(pages_dict["ts"]),
                    pages_dict["url"],
                )

            # Add the key to the dictionary with a blank value
            passed_pages_dict[key] = {}

            # If title was in the passed pages line add it to the value of the just created dictionary entry
            if "title" in pages_dict:
                passed_pages_dict[key]["title"] = pages_dict["title"]

            # If text was in the passed pages line add it to the value of the just created dictionary entry
            if "text" in pages_dict:
                passed_pages_dict[key]["text"] = pages_dict["text"]

    return passed_pages_dict


def now():
    """Returns the current time"""
    return tuple(datetime.datetime.utcnow().timetuple()[:6])


def validateJSON(jsonData):
    """Attempts to validate a string as json"""
    try:
        json.loads(jsonData)
    except ValueError as err:
        return False
    return True



import unittest
import tempfile
import os
from wacz.main import main, now
import zipfile

TEST_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures")


class TestWaczIndexing(unittest.TestCase):
    def test_warc_with_other_metadata(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-warcinfo-metadata.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-warcinfo-metadata.wacz"),
                    ]
                ),
                0,
            )

            self.assertEqual(
                main(
                    [
                        "validate",
                        "-f",
                        os.path.join(tmpdir, "example-warcinfo-metadata.wacz"),
                    ]
                ),
                0,
            )

    def test_warc_with_extra_lists(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection-with-lists.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-with-lists.wacz"),
                    ]
                ),
                0,
            )

            self.assertEqual(
                main(
                    [
                        "validate",
                        "-f",
                        os.path.join(tmpdir, "example-collection-with-lists.wacz"),
                    ]
                ),
                0,
            )

            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-with-lists.wacz")
            ) as zf:
                filelist = sorted(zf.namelist())

                # verify pages file added for each list
                self.assertEqual(
                    filelist,
                    [
                        "archive/example-collection-with-lists.warc",
                        "datapackage.json",
                        "indexes/index.cdx.gz",
                        "indexes/index.idx",
                        "pages/example.jsonl",
                        "pages/iana.jsonl",
                        "pages/pages.jsonl",
                    ],
                )



import tempfile, os, zipfile, json, pathlib, pkg_resources, gzip
from frictionless import validate
from wacz.util import support_hash_file, now
from wacz.waczindexer import WACZIndexer
from io import BytesIO, StringIO, TextIOWrapper
import glob

OUTDATED_WACZ = "0.1.0"


class Validation(object):
    def __init__(self, file):
        self.dir = tempfile.TemporaryDirectory()
        self.wacz = file
        with zipfile.ZipFile(file, "r") as zip_ref:
            zip_ref.extractall(self.dir.name)
            zip_ref.close()
        self.detect_version()
        self.detect_hash_type()

    def check_required_contents(self):
        """Checks the general component of the wacz and notifies users whats missing"""
        if os.path.exists(os.path.join(self.dir.name, "datapackage.json")) is False:
            print("Datapackage is missing from your wacz file")
            return 1
        if (
            glob.glob(os.path.join(self.dir.name, "archive/*.warc")) == False
            and glob.glob(os.path.join(self.dir.name, "archive/*.warc.gz")) == False
        ):
            print(
                "A warc file is missing from your archive folder you must have a .warc or .warc.gz file in your archive folder"
            )
            return 1
        if (
            glob.glob(os.path.join(self.dir.name, "indexes/index.cdx.gz")) == False
            and glob.glob(os.path.join(self.dir.name, "indexes/index.cdx.gz")) == False
            and glob.glob(os.path.join(self.dir.name, "indexes/index.idx")) == False
        ):
            print(
                "An index file is missing from your indexes folder you must have an index.cdx.gz, index,cdx or index.idx in your index folder"
            )
            return 1
        if glob.glob(os.path.join(self.dir.name, "pages/pages.jsonl")) == False:
            print(
                "An index file is missing from your indexes folder you must have an index.cdx.gz, index,cdx or index.idx in your index folder"
            )
            return 1

        return 0

    def detect_hash_type(self):
        self.hash_type = None
        # we know the datapackage exists at this point because we're running it after the version check
        self.datapackage_path = os.path.join(self.dir.name, "datapackage.json")
        self.datapackage = json.loads(open(self.datapackage_path, "rb").read())
        try:
            self.hash_type = self.datapackage["resources"][0]["hashing"]
            return 0
        except:
            print(
                "\nHashing type was unable to be detected the wacz file may have no resources"
            )
            return 1

    def detect_version(self):
        self.version = None
        if os.path.exists(os.path.join(self.dir.name, "datapackage.json")):
            self.data_folder = os.listdir(self.dir.name)
            self.datapackage_path = os.path.join(self.dir.name, "datapackage.json")
            self.datapackage = json.loads(open(self.datapackage_path, "rb").read())

            try:
                self.version = self.datapackage["wacz_version"]
            except:
                print("\nVersion missing from datapackage.json, invalid wacz file")
                return

            print("\nVersion detected as %s" % self.version)
        elif os.path.exists(os.path.join(self.dir.name, "webarchive.yaml")):
            self.version = OUTDATED_WACZ
            self.webarchive_yaml = os.path.join(self.dir.name, "webarchive.yaml")
            print(
                "\nWACZ version detected as 0.1.0. This is an outdated version of WACZ."
            )
        else:
            print("\nVersion not able to be detected, invalid wacz file")

    def frictionless_validate(self):
        """Uses the frictionless data package to validate the datapackage.json file"""
        if validate(self.datapackage_path).valid == True:
            return True
        else:
            print(
                "\nFrictionless has detected that this is an invalid package with errors %s"
                % validate(self.datapackage_path).errors
            )
            return False

    def check_file_paths(self):
        """Uses the datapackage to check that all the files listed exist in the data folder or that the wacz contains a webarchive.yml file"""
        if self.version != OUTDATED_WACZ:
            package_files = [item["path"] for item in self.datapackage["resources"]]
            for filepath in pathlib.Path(self.dir.name).glob("**/*.*"):
                if not os.path.basename(filepath).endswith("datapackage.json"):
                    file = str(filepath).split("/")[-2:]
                    file = "/".join(file)
                    if file not in package_files:
                        print("file %s is not listed in the datapackage" % file)
                        return False
        return True

    def check_compression(self):
        """WARCs and compressed cdx.gz should be in ZIP with 'store' compression (not deflate) Indexes and page list can be compressed"""
        wacz = zipfile.ZipInfo(self.wacz)
        if wacz.compress_type != 0:
            return False

        if os.path.exists(os.path.join(self.dir.name, "indexes/index.cdx.gz")):
            cdx = zipfile.ZipInfo(os.path.join(self.dir.name, "indexes/index.cdx.gz"))
            if cdx.compress_type != 0:
                return False

        archive_folder = os.listdir(os.path.join(self.dir.name, "archive"))
        for item in archive_folder:
            if ".warc" not in item and zf.getinfo(item).compress_type != 0:
                return False
        return True

    def check_indexes(self):
        """Indexing existing WARC which should match the index in the wacz"""
        if os.path.exists(os.path.join(self.dir.name, "indexes/index.cdx.gz")):
            for resource in self.datapackage["resources"]:
                if resource["path"] == "indexes/index.cdx.gz":
                    cdx = resource["stats"]["hash"]
        else:
            return False

        archive_folder = os.listdir(os.path.join(self.dir.name, "archive"))
        for item in archive_folder:
            if ".warc" in item:
                warc = item
        wacz_file = tempfile.NamedTemporaryFile(delete=False)
        wacz = zipfile.ZipFile(wacz_file.name, "w")
        data_file = zipfile.ZipInfo("indexes/index.cdx.gz", now())
        index_buff = BytesIO()
        text_wrap = TextIOWrapper(index_buff, "utf-8", write_through=True)
        wacz_indexer = None
        with wacz.open(data_file, "w") as data:
            wacz_indexer = WACZIndexer(
                text_wrap,
                {},
                sort=True,
                compress=data,
                fields="referrer",
                data_out_name="index.cdx.gz",
                records="all",
                main_url="",
                detect_pages="",
            )

            wacz_indexer.process_all()
        wacz.close()
        dir = tempfile.TemporaryDirectory()
        with zipfile.ZipFile(self.wacz, "r") as zip_ref:
            zip_ref.extractall(dir.name)
            zip_ref.close()

        with open(os.path.join(dir.name, "indexes/index.cdx.gz"), "rb") as fd:
            hash = support_hash_file(self.hash_type, fd.read())
            gzip_fd = gzip.GzipFile(fileobj=fd)

        return cdx == hash

    def check_file_hashes(self):
        """Uses the datapackage to check that all the hashes of file in the data folder match those in the datapackage"""
        for filepath in pathlib.Path(self.dir.name).glob("**/*.*"):
            if not os.path.basename(filepath).endswith("datapackage.json"):
                file = open(filepath, "rb").read()
                hash = support_hash_file(self.hash_type, file)
                file = str(filepath).split("/")[-2:]
                file = "/".join(file)
                res = None
                for item in self.datapackage["resources"]:
                    if item["path"] == file:
                        res = item
                if res == None or (res["stats"]["hash"] != hash):
                    print(
                        "\nfile %s's hash does not match the has listed in the datapackage"
                        % file
                    )
                    return False
        return True



import unittest
import tempfile
import os
import zipfile, json, gzip, hashlib
from wacz.util import support_hash_file, validateJSON

TEST_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures")


class TestUtilFunctions(unittest.TestCase):
    def test_util_hash(self):
        """When invoking the util hash method a  hash should be returned"""
        test_hash = hashlib.sha256("test".encode("utf-8")).hexdigest()
        self.assertEqual(support_hash_file("sha256", "test".encode("utf-8")), test_hash)

        test_hash = hashlib.md5("test".encode("utf-8")).hexdigest()
        self.assertEqual(support_hash_file("md5", "test".encode("utf-8")), test_hash)

    def test_util_validate_json_succeed(self):
        """validate json method should succed with valid json"""
        self.assertTrue(validateJSON('{"test": "test"}'))

    def test_util_validate_json_fail(self):
        """validate json method should fail with valid json"""
        self.assertFalse(validateJSON('test": "test"}'))


if __name__ == "__main__":
    unittest.main()



import unittest
import tempfile
import os
import zipfile, json, gzip
from wacz.main import main, now
from unittest.mock import patch
import jsonlines

TEST_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures")


class TestWaczFormat(unittest.TestCase):
    def test_warc_with_invalid_passed_pages(self):
        """If a user passes an invalid file using --page we should return an error"""
        with tempfile.TemporaryDirectory() as tmpdir:
            fp = tempfile.NamedTemporaryFile()
            fp.write(
                """{"format": "title": "All Pages"}\n{"http://www.example"  "0-10-07T21:22:36Z", "title": "Example Domain"}""".encode(
                    "utf-8"
                )
            )
            fp.seek(0)
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-url.wacz"),
                        "-p",
                        os.path.join(tmpdir, fp.name),
                    ]
                ),
                1,
            )

    @patch("wacz.main.now")
    def test_warc_with_pages_flag(self, mock_now):
        """When passing the pages flag with a valid pages.jsonl file a pages/pages.jsonl file should be created"""
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)

        with tempfile.TemporaryDirectory() as tmpdir:
            fp = tempfile.NamedTemporaryFile()
            fp.write(
                """{"format": "json-pages-1.0", "id": "pages", "title": "All Pages"}\n{"id": "1db0ef709a", "url": "http://www.example.com/", "ts": "2020-10-07T21:22:36Z", "title": "Example Domain"}""".encode(
                    "utf-8"
                )
            )
            fp.seek(0)
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-url.wacz"),
                        "-p",
                        os.path.join(tmpdir, fp.name),
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-url.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_pages"))
                zip_ref.close()

            self.assertEqual(
                main(
                    [
                        "validate",
                        "-f",
                        os.path.join(tmpdir, "example-collection-valid-url.wacz"),
                    ]
                ),
                0,
            )
            wacz_pages = os.path.join(tmpdir, "unzipped_valid_pages/pages/pages.jsonl")
            wacz_cdx = os.path.join(tmpdir, "unzipped_valid_pages/indexes/index.cdx.gz")
            cdx_content = gzip.open(wacz_cdx, "rb").read()
            self.assertTrue(
                "pages.jsonl"
                in os.listdir(os.path.join(tmpdir, "unzipped_valid_pages/pages/"))
            )
            with open(wacz_pages) as f:
                for _ in range(1):
                    next(f)
                for line in f:
                    obj = json.loads(line)
                    self.assertTrue("id" in obj.keys())
                    self.assertTrue("ts" in obj.keys())
                    self.assertTrue("url" in obj.keys())
                    self.assertTrue(obj["url"].encode() in cdx_content)

    @patch("wacz.main.now")
    def test_warc_with_detect_pages_flag(self, mock_now):
        """When passing the text index flag pages/pages.jsonl should be generated."""
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-url.wacz"),
                        "--detect-pages",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-url.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_pages"))
                zip_ref.close()

            wacz_pages = os.path.join(tmpdir, "unzipped_valid_pages/pages/pages.jsonl")
            wacz_cdx = os.path.join(tmpdir, "unzipped_valid_pages/indexes/index.cdx.gz")
            cdx_content = gzip.open(wacz_cdx, "rb").read()
            self.assertTrue(
                "pages.jsonl"
                in os.listdir(os.path.join(tmpdir, "unzipped_valid_pages/pages/"))
            )
            with open(wacz_pages) as f:
                for _ in range(1):
                    next(f)
                for line in f:
                    obj = json.loads(line)
                    self.assertTrue("id" in obj.keys())
                    self.assertTrue("ts" in obj.keys())
                    self.assertTrue("title" in obj.keys())
                    self.assertTrue("url" in obj.keys())
                    self.assertTrue(obj["url"].encode() in cdx_content)

    @patch("wacz.main.now")
    def test_warc_with_text_index_flag(self, mock_now):
        """When passing the text index flag pages/pages.jsonl should be generated with a full and accurate text index."""
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-url.wacz"),
                        "-t",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-url.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_text"))
                zip_ref.close()

            wacz_pages = os.path.join(tmpdir, "unzipped_valid_text/pages/pages.jsonl")
            wacz_cdx = os.path.join(tmpdir, "unzipped_valid_text/indexes/index.cdx.gz")
            cdx_content = gzip.open(wacz_cdx, "rb").read()
            self.assertTrue(
                "pages.jsonl"
                in os.listdir(os.path.join(tmpdir, "unzipped_valid_text/pages/"))
            )
            with open(wacz_pages) as f:
                for _ in range(1):
                    next(f)
                for line in f:
                    obj = json.loads(line)
                    self.assertTrue("id" in obj.keys())
                    self.assertTrue("ts" in obj.keys())
                    self.assertTrue("title" in obj.keys())
                    self.assertTrue("url" in obj.keys())
                    self.assertTrue(obj["url"].encode() in cdx_content)
                    self.assertTrue("text" in obj.keys())

    def test_warc_with_both_p_and_d_flag(self):
        """If a user passes both the --pages and --detect-pages flags we should return an error and a message about needing only one"""
        with tempfile.TemporaryDirectory() as tmpdir:
            with self.assertRaises(SystemExit):
                self.assertEqual(
                    main(
                        [
                            "create",
                            "-f",
                            os.path.join(TEST_DIR, "example-collection.warc"),
                            "-o",
                            os.path.join(tmpdir, "example-collection.wacz"),
                            "--detect_pages",
                            "-p",
                            "test.jsonl",
                        ]
                    ),
                    0,
                )

    def test_warc_with_only_ts_flag(self):
        """If a user only passes the --ts flag we should return an error and a message about needing to also pass the --url flag"""
        with tempfile.TemporaryDirectory() as tmpdir:
            with self.assertRaises(SystemExit):
                self.assertEqual(
                    main(
                        [
                            "create",
                            "-f",
                            os.path.join(TEST_DIR, "example-collection.warc"),
                            "-o",
                            os.path.join(tmpdir, "example-collection.wacz"),
                            "--ts",
                            "2020104212236",
                        ]
                    ),
                    0,
                )

    @patch("wacz.main.now")
    def test_warc_with_valid_date_flag(self, mock_now):
        """When passing a valid date flag the datapackage should have that as the mainpageTS"""
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-desc.wacz"),
                        "--desc",
                        "fake desc",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-desc.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_desc"))
                zip_ref.close()

            self.wacz_json = os.path.join(
                tmpdir, "unzipped_valid_desc/datapackage.json"
            )
            self.wacz_pages = os.path.join(
                tmpdir, "unzipped_valid_desc/pages/pages.jsonl"
            )

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            self.assertEqual(json_parse["des"], "fake desc")

    @patch("wacz.main.now")
    def test_warc_with_valid_date_flag(self, mock_now):
        """When passing a valid title flag the datapackage should have that as the title value"""
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-title.wacz"),
                        "--title",
                        "Example Title",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-title.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_title"))
                zip_ref.close()

            self.wacz_json = os.path.join(
                tmpdir, "unzipped_valid_title/datapackage.json"
            )
            self.wacz_pages = os.path.join(
                tmpdir, "unzipped_valid_title/pages/pages.jsonl"
            )

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            self.assertEqual(json_parse["title"], "Example Title")

    @patch("wacz.main.now")
    def test_warc_with_valid_date_flag(self, mock_now):
        """When passing a valid date flag the datapackage should have that as the mainpageTS"""
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-date.wacz"),
                        "--date",
                        "2020-11-01",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-date.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_date"))
                zip_ref.close()

            self.wacz_json = os.path.join(
                tmpdir, "unzipped_valid_date/datapackage.json"
            )
            self.wacz_pages = os.path.join(
                tmpdir, "unzipped_valid_date/pages/pages.jsonl"
            )

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            self.assertEqual(json_parse["metadata"]["mainPageTS"], "2020-11-01")

    @patch("wacz.main.now")
    def test_warc_with_valid_url_flag(self, mock_now):
        """When passing a valid url flag the url should be added to the pages.jsonl file and appear in the datapackage"""
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-valid-url.wacz"),
                        "--url",
                        "http://www.example.com/",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-url.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_url"))
                zip_ref.close()

            self.wacz_json = os.path.join(tmpdir, "unzipped_valid_url/datapackage.json")
            self.wacz_pages = os.path.join(
                tmpdir, "unzipped_valid_url/pages/pages.jsonl"
            )

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            f = open(self.wacz_pages, "rb")
            json_pages = [json.loads(jline) for jline in f.read().splitlines()]
            self.assertEqual(json_pages[1]["url"], "http://www.example.com/")
            self.assertEqual(
                json_parse["metadata"]["mainPageURL"], "http://www.example.com/"
            )
            assert "mainPageTS" not in json_parse.keys()

    def test_warc_with_invalid_url_flag(self):
        """When passing an invalid url flag we should raise a ValueError"""
        with tempfile.TemporaryDirectory() as tmpdir:
            with self.assertRaises(ValueError):
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection.wacz"),
                        "--url",
                        "http://www.examplefake.com/",
                    ]
                )

    def test_warc_with_valid_url_and_invalid_ts_flag(self):
        """When passing a valid url flag with an invalid ts flag we should raise a ValueError"""
        with tempfile.TemporaryDirectory() as tmpdir:
            with self.assertRaises(ValueError):
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection.wacz"),
                        "--url",
                        "http://www.example.com/",
                        "--ts",
                        "2020104212236",
                    ]
                )

    @patch("wacz.main.now")
    def test_warc_with_valid_url_and_ts_flag(self, mock_now):
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        """When passing an a valid url and ts flag we should see those values represented in the datapackage and pages.jsonl file"""
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(
                            tmpdir, "example-collection-valid-url-valid-ts.wacz"
                        ),
                        "--url",
                        "http://www.example.com/",
                        "--ts",
                        "20201007212236",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-valid-url-valid-ts.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_valid_url_valid_ts"))
                zip_ref.close()

            self.wacz_json = os.path.join(
                tmpdir, "unzipped_valid_url_valid_ts/datapackage.json"
            )
            self.wacz_pages = os.path.join(
                tmpdir, "unzipped_valid_url_valid_ts/pages/pages.jsonl"
            )

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            f = open(self.wacz_pages, "rb")
            json_pages = [json.loads(jline) for jline in f.read().splitlines()]
            self.assertEqual(json_pages[1]["url"], "http://www.example.com/")
            self.assertEqual(
                json_parse["metadata"]["mainPageURL"], "http://www.example.com/"
            )
            self.assertEqual(json_parse["metadata"]["mainPageTS"], "20201007212236")

    @patch("wacz.main.now")
    def test_warc_with_hash_flag_md5(self, mock_now):
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        """When passing the --hash-type flag with a value of md5 the datapackage should be hashed using md5"""
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-md5.wacz"),
                        "--hash-type",
                        "md5",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-md5.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_md5"))
                zip_ref.close()

            self.wacz_json = os.path.join(tmpdir, "unzipped_md5/datapackage.json")

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            self.assertEqual(json_parse["resources"][0]["hashing"], "md5")

    @patch("wacz.main.now")
    def test_warc_with_hash_flag_sha256(self, mock_now):
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        """When passing the --hash-type flag with a value of sha256 the datapackage should be hashed using sha256"""
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-sha256.wacz"),
                        "--hash-type",
                        "sha256",
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-sha256.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_sha256"))
                zip_ref.close()

            self.wacz_json = os.path.join(tmpdir, "unzipped_sha256/datapackage.json")

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            self.assertEqual(json_parse["resources"][0]["hashing"], "sha256")

    @patch("wacz.main.now")
    def test_warc_with_hash_flag_sha256(self, mock_now):
        mock_now.return_value = (2020, 10, 7, 22, 29, 10)
        """When not passing the --hash-type flag the datapackage should be hashed using sha256"""
        with tempfile.TemporaryDirectory() as tmpdir:
            self.assertEqual(
                main(
                    [
                        "create",
                        "-f",
                        os.path.join(TEST_DIR, "example-collection.warc"),
                        "-o",
                        os.path.join(tmpdir, "example-collection-sha256.wacz"),
                    ]
                ),
                0,
            )
            with zipfile.ZipFile(
                os.path.join(tmpdir, "example-collection-sha256.wacz"), "r"
            ) as zip_ref:
                zip_ref.extractall(os.path.join(tmpdir, "unzipped_sha256"))
                zip_ref.close()

            self.wacz_json = os.path.join(tmpdir, "unzipped_sha256/datapackage.json")

            f = open(self.wacz_json, "rb")
            json_parse = json.loads(f.read())

            self.assertEqual(json_parse["resources"][0]["hashing"], "sha256")


if __name__ == "__main__":
    unittest.main()
